package com.huimv.eartag2.eartag.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.huimv.eartag2.common.dao.entity.EartagDataEntity;
import com.huimv.eartag2.common.dao.entity.EartagEartagOnlineEntity;
import com.huimv.eartag2.common.dao.entity.EartagEartagRegisterEntity;
import com.huimv.eartag2.common.dao.entity.EartarFarmAllStatusEntity;
import com.huimv.eartag2.common.dao.repo.EartagDataRepo;
import com.huimv.eartag2.common.dao.repo.EartagEartagOnlineRepo;
import com.huimv.eartag2.common.dao.repo.EartagEartagRegisterRepo;
import com.huimv.eartag2.common.dao.repo.EartarFarmAllStatusRepo;
import com.huimv.eartag2.common.utils.DateUtil;
import com.huimv.eartag2.eartag.service.ICacheService;
import com.huimv.eartag2.eartag.service.IDeviceService;
import com.huimv.eartag2.eartag.service.IEartagService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.sql.Timestamp;
import java.text.ParseException;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
@Slf4j
public class EartagServiceImpl implements IEartagService {
    @Autowired
    private RedisTemplate redisTemplate;
    @Autowired
    private EartagDataRepo eartagDataRepo;
    @Autowired
    private ICacheService cacheService;
    @Autowired
    private IDeviceService deviceService;
    @Autowired
    private EartagEartagRegisterRepo eartagRegisterRepo;
    @Autowired
    private EartagEartagOnlineRepo eartagOnlineRepo;
    @Autowired
    private EartarFarmAllStatusRepo farmAllStatusRepo;

    @Override
    public void handleEartag(JSONObject dataJo) throws ParseException {
        //保存耳标流水
        saveEartag(dataJo);

        //设备编码
        String deviceCode = dataJo.getString("device");
//        System.out.println("444 设备编号>>" + deviceCode);
        //耳标号
        String earmark = dataJo.getString("earmark");
//        System.out.println("444 耳标号>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>" + earmark);

        //--保存设备注册数据(MySQL,Redis)
        //--保存设备状态数据(MySQL,Redis)
        String todayDateText = new DateUtil().getTodayDateText();
        java.sql.Date todayDate = new java.sql.Date(new java.util.Date().getTime());
        Timestamp nowTimestamp = new Timestamp(new java.util.Date().getTime());

//        System.out.println("## eartagOnlineSetExpire>>"+eartagOnlineSetExpire);
        //--设备在线统计表
        //--设备状态每日统计表
        //--设备和耳标总状态表
        //获取牧场id
        String farmId = deviceService.getFarmIdByDeviceCode(deviceCode);

        //Step1:更新耳标注册数据
        updateEartagRegister(earmark,deviceCode,nowTimestamp,farmId);

        //Step2:更新耳标在线数据
        updateEartagOnline(earmark,deviceCode,todayDateText,todayDate,nowTimestamp,dataJo,farmId);

        //判断该设备编号是否存在牧场缓存在线集合当中；
        if (!cacheService.isExistEartagOnlineSet(farmId, earmark,todayDateText)) {

            //更新总状态数据
            updateEartagOfAllStatus(earmark,deviceCode,todayDateText,farmId);

            //将耳标号加入到耳标在线集合当中
            cacheService.putEartagToOnlineInSet(farmId, earmark,todayDateText);
        }
    }

    //状态更新
    public void updateEartagOfAllStatus(String earmark, String deviceCode,String todayDateText, String farmId) {
        //# 判断设备和耳标总状态表是否存在
        //      存在：更新在线耳标数量；
        //      不存在：创建状态
        EartarFarmAllStatusEntity farmAllStatusEntity = farmAllStatusRepo.getOneByFarmIdAndCreateDate(farmId,todayDateText);
        if(farmAllStatusEntity == null){
            //#获取耳标注册数量#
            int eartagTotal =  getEartagRegTotalByFarmId(farmId);
            //#总状态记录不存在；
            EartarFarmAllStatusEntity newFarmAllStatusEntity = new EartarFarmAllStatusEntity();
            newFarmAllStatusEntity.setEartagTotal(eartagTotal);
            newFarmAllStatusEntity.setEartagOnline(1);
            farmAllStatusRepo.saveAndFlush(newFarmAllStatusEntity);
        }else{
            //#存在；
            int total = farmAllStatusEntity.getEartagOnline();
            farmAllStatusEntity.setEartagOnline(++total);
            farmAllStatusRepo.saveAndFlush(farmAllStatusEntity);
        }
    }

    //获取耳标注册数量
    public Integer getEartagRegTotalByFarmId(String farmId){
        int eartagTotal = 0;
        List<Object[]> eartagRegList = eartagRegisterRepo.getEartagCountByFarmId(farmId);
        Object[] eartagRegObj = (Object[]) eartagRegList.get(0);
        //处于工作状态的耳标总数
        return Integer.parseInt(eartagRegObj[0].toString());
    }

    //更新耳标在线数据
    public void updateEartagOnline(String earmark, String deviceCode, String todayDateText, java.sql.Date todayDate, Timestamp nowTimestamp, JSONObject eartagJo, String farmId) {
        //# 判断耳标在线统计数据是否存在
        //      存在，则更新上传记录数、最后上传时间、环境温度，耳根温度，并将上面数据更新到缓存数据中；
        //      不存在，则创建耳标在线记录，并更新到缓存数据中；
        //耳根温度
        String earTemp =  eartagJo.getString("earTemp1");
        //环境温度
        String envTemp =  eartagJo.getString("envTemp");
        EartagEartagOnlineEntity eartagOnlineEntity = eartagOnlineRepo.getOneByEarmarkAndFarmId(earmark,todayDateText,farmId);
        if(eartagOnlineEntity == null){
            //# 不存在
            EartagEartagOnlineEntity newEartagOnlineEntity = new EartagEartagOnlineEntity();
            newEartagOnlineEntity.setEarmark(earmark);
            newEartagOnlineEntity.setTotal(1);
            newEartagOnlineEntity.setFirstTime(nowTimestamp);
            newEartagOnlineEntity.setLastTime(nowTimestamp);
            newEartagOnlineEntity.setEnvTemp(envTemp);
            newEartagOnlineEntity.setEartagTemp(earTemp);
            newEartagOnlineEntity.setAddDate(todayDate);
            newEartagOnlineEntity.setFarmId(farmId);
            eartagOnlineRepo.saveAndFlush(newEartagOnlineEntity);
            //更新耳标在线统计
            cacheService.putEartagOnlineCount(earmark,newEartagOnlineEntity);
        }else{
            //# 存在
            int total = eartagOnlineEntity.getTotal();
            eartagOnlineEntity.setTotal(++total);
            eartagOnlineEntity.setLastTime(nowTimestamp);
            eartagOnlineEntity.setEnvTemp(envTemp);
            eartagOnlineEntity.setEartagTemp(earTemp);
            eartagOnlineRepo.saveAndFlush(eartagOnlineEntity);
            //更新耳标在线统计
            cacheService.putEartagOnlineCount(earmark,eartagOnlineEntity);
        }
    }

    //更新耳标注册数据
    public void updateEartagRegister(String earmark, String deviceCode, Timestamp nowTimestamp, String farmId) {
        //# 判断耳标注册表是否存在；
        //      如果不存在就注册耳标；
        //      如果存在就更新活动状态（active_status）和活动状态更新时间（active_time）
        //
        EartagEartagRegisterEntity eartagRegisterEntity = eartagRegisterRepo.getOneByEarmark(earmark,farmId);
        if(eartagRegisterEntity == null){
            //#创建耳标注册记录#
            EartagEartagRegisterEntity newEartagRegisterEntity = new EartagEartagRegisterEntity();
            newEartagRegisterEntity.setEarmark(earmark);
            newEartagRegisterEntity.setFirstTime(nowTimestamp);
            newEartagRegisterEntity.setLastTime(nowTimestamp);
            newEartagRegisterEntity.setFirstDevice(deviceCode);
            newEartagRegisterEntity.setBelongDevice(deviceCode);
            newEartagRegisterEntity.setLastDevice(deviceCode);
            newEartagRegisterEntity.setRegisterTime(nowTimestamp);
            newEartagRegisterEntity.setRegisterType(1);
            newEartagRegisterEntity.setFarmId(farmId);
            newEartagRegisterEntity.setActiveStatus(1);
            newEartagRegisterEntity.setActiveTime(nowTimestamp);
            newEartagRegisterEntity.setLiveStatus(1);
            eartagRegisterRepo.saveAndFlush(newEartagRegisterEntity);
            //更新耳标注册消息缓存
            cacheService.putEartagRegister(earmark,newEartagRegisterEntity);
        }else{
            eartagRegisterEntity.setActiveStatus(1);
            eartagRegisterEntity.setActiveTime(nowTimestamp);
            eartagRegisterRepo.saveAndFlush(eartagRegisterEntity);
            //更新耳标注册消息缓存
            cacheService.putEartagRegister(earmark,eartagRegisterEntity);
        }
    }

    /**
     * @Method : countAct
     * @Description : 计算act数值
     * @Params : [deviceCode, nowAct0]
     * @Return : java.lang.Integer
     * @Author : ZhuoNing
     * @Date : 2022/1/18
     * @Time : 15:25
     */
    @Override
    public Integer countAct(String deviceCode, String nowAct) {
        //读取hash
        Object actObj = redisTemplate.opsForHash().get(deviceCode, "act");
        if (actObj == null) {
            //--初始化最新的redis记录
            initRedisObj(deviceCode);
            Object lastAct = redisTemplate.opsForHash().get(deviceCode, "act");
            if (lastAct == null) {
                log.error("#--- redis数据库有问题，请检查redis是否能正常连接 ---# ");
                return 0;
            }
        }
        Object lastAct = redisTemplate.opsForHash().get(deviceCode, "act");
        Integer act1 = Integer.parseInt(nowAct) - Integer.parseInt(lastAct.toString());
//        redisTemplate.opsForHash().put(deviceCode, "act1", act1);
        return act1;
    }

    /**
     * @Author      : ZhuoNing
     * @Date        : 2022/2/9
     * @Time        : 14:33
     */
    @Override
    public void saveEartag(JSONObject eartagJo) {
        EartagDataEntity dataEntity = new EartagDataEntity();
        dataEntity.setCmdHeader(eartagJo.getString("cmdHeader"));
        dataEntity.setDevice(eartagJo.getString("device"));
        dataEntity.setEarmark(eartagJo.getString("earmark"));
        dataEntity.setBat(Integer.parseInt(eartagJo.getString("bat")));
        dataEntity.setEarTemp(Integer.parseInt(eartagJo.getString("earTemp")));
        dataEntity.setEarTemp1(eartagJo.getFloat("earTemp1"));
        dataEntity.setEnvTemp(Integer.parseInt(eartagJo.getString("envTemp")));
        dataEntity.setEnvTemp1(Integer.parseInt(eartagJo.getString("envTemp")));
        dataEntity.setAct(Integer.parseInt(eartagJo.getString("act")));
        dataEntity.setAct1(Integer.parseInt(eartagJo.getString("act1")));
        dataEntity.setSignal1(Integer.parseInt(eartagJo.getString("signal")));
        dataEntity.setAskTime(getAskTime(eartagJo.getString("askTime")));
        dataEntity.setOther(eartagJo.getString("other"));
        dataEntity.setAddTime(new Timestamp(new Date().getTime()));
        eartagDataRepo.saveAndFlush(dataEntity);

        //最新的耳标数据覆盖老数据
        Map<String, String> map = new HashMap<>();
        map.put("cmdHeader", dataEntity.getCmdHeader());
        map.put("device", dataEntity.getDevice());
        map.put("earmark", dataEntity.getEarmark());
        map.put("bat", dataEntity.getBat().toString());
        map.put("earTemp", dataEntity.getEarTemp().toString());
        map.put("earTemp1", dataEntity.getEarTemp1().toString());
        map.put("envTemp", dataEntity.getEnvTemp().toString());
        map.put("envTemp1", dataEntity.getEnvTemp1().toString());
        map.put("act", dataEntity.getAct().toString());
        map.put("act1", dataEntity.getAct1().toString());
        map.put("signal1", dataEntity.getSignal1().toString());
        map.put("askTime", dataEntity.getAskTime());
        map.put("other", dataEntity.getOther());
        map.put("addTime", dataEntity.getAddTime().toString());
        //为hash结构设置多个键值对(hmset)
        redisTemplate.opsForHash().putAll(dataEntity.getDevice(), map);
    }

    //
    public String getAskTime(String askTime) {
        String newAskTime = askTime.substring(0, 4) + "-" + askTime.substring(4, 6) + "-" + askTime.substring(6, 8) + " " + askTime.substring(8, 10) + ":" + askTime.substring(10, 12) + ":" + askTime.substring(12, 14);
        return newAskTime;
    }

    /**
     * 初始化redis最新记录
     */
    private void initRedisObj(String deviceCode) {
        EartagDataEntity lastEartagDataEntity = eartagDataRepo.getLastEartagData(deviceCode);
        if (lastEartagDataEntity != null) {
            System.out.println("## lastEartagDataEntity.toString>>" + lastEartagDataEntity.toString());
            //--从数据库读取最新记录并覆盖redis
            Map<String, String> map = new HashMap<>();
            map.put("cmdHeader", lastEartagDataEntity.getCmdHeader());
            map.put("device", lastEartagDataEntity.getDevice());
            map.put("earmark", lastEartagDataEntity.getEarmark());
            map.put("bat", lastEartagDataEntity.getBat().toString());
            map.put("earTemp", lastEartagDataEntity.getEarTemp().toString());
            map.put("earTemp1", lastEartagDataEntity.getEarTemp1().toString());
            map.put("envTemp", lastEartagDataEntity.getEnvTemp().toString());
            map.put("envTemp1", lastEartagDataEntity.getEnvTemp1().toString());
            map.put("act", lastEartagDataEntity.getAct().toString());
            map.put("act1", lastEartagDataEntity.getAct1().toString());
            map.put("signal1", lastEartagDataEntity.getSignal1().toString());
            map.put("askTime", lastEartagDataEntity.getAskTime());
            map.put("other", lastEartagDataEntity.getOther());
            map.put("addTime", lastEartagDataEntity.getAddTime().toString());
            //为hash结构设置多个键值对(hmset)
            redisTemplate.opsForHash().putAll(deviceCode, map);
        }
    }
}
